import warnings


def iniformat(s: str, /) -> str:
    from inifix._format import format_string

    warnings.warn(
        "inifix.format.iniformat is deprecated since v5.0.0 "
        "and will be removed in a future version. "
        "Please use inifix.format_string instead.",
        category=DeprecationWarning,
        stacklevel=2,
    )
    return format_string(s)



import re
import sys
from enum import Enum, auto

if sys.version_info >= (3, 11):
    from typing import assert_never
else:
    from typing_extensions import assert_never

__all__ = ["FloatEncoder"]


class FloatEncoder(Enum):
    SIMPLE = auto()
    ENOTATION = auto()
    ENOTATION_IFF_SHORTER = auto()

    @staticmethod
    def _simplify(s: str, /) -> str:
        """
        Simplify exponents and trailing zeros in decimals.
        """
        s = re.sub(r"\.?0*(e[+-]?)0", r"\1", s)
        s = re.sub(r"(e-0)$", "e0", s)
        return s.replace("+", "")

    def encode(self, r: float, /) -> str:
        match self:
            case FloatEncoder.SIMPLE:
                return str(float(r))
            case FloatEncoder.ENOTATION:
                base = str(r)
                if "e" in base:
                    return self._simplify(base)
                if not base.strip(".0"):
                    return "0e0"
                max_ndigit = len(base.replace(".", "")) - 1
                return self._simplify(f"{r:.{max_ndigit}e}")
            case FloatEncoder.ENOTATION_IFF_SHORTER:
                return min(
                    FloatEncoder.SIMPLE.encode(r),
                    FloatEncoder.ENOTATION.encode(r),
                    key=lambda x: len(x),
                )
            case _ as unreachable:
                assert_never(unreachable)



from pathlib import Path

import pytest

DATA_DIR = Path(__file__).parent / "data"
INIFILES_PATHS = list(DATA_DIR.glob("*.ini")) + list(DATA_DIR.glob("*.cfg"))
INIFILES_IDS = [inifile.name[:-4] for inifile in INIFILES_PATHS]

INIFILES = dict(zip(INIFILES_PATHS, INIFILES_IDS, strict=True))

INIFILES_W_SECTIONS = {
    path: id
    for path, id in INIFILES.items()
    if "fargo" not in id and path.suffix != ".cfg"
}
INIFILES_WO_SECTIONS = {
    path: id for path, id in INIFILES.items() if path not in INIFILES_W_SECTIONS
}


@pytest.fixture()
def datadir():
    return DATA_DIR


@pytest.fixture(params=list(INIFILES.keys()), ids=list(INIFILES.values()))
def inifile(request):
    return request.param


@pytest.fixture(
    params=list(INIFILES_W_SECTIONS.keys()), ids=list(INIFILES_W_SECTIONS.values())
)
def inifile_with_sections(request):
    return request.param


@pytest.fixture(
    params=list(INIFILES_WO_SECTIONS.keys()), ids=list(INIFILES_WO_SECTIONS.values())
)
def inifile_without_sections(request):
    return request.param



from math import isnan

import pytest
from hypothesis import example, given
from hypothesis import strategies as st

from inifix._floatencoder import FloatEncoder


@pytest.mark.parametrize(
    "input, expected",
    [
        ("1e-00", "1e0"),
        ("1e-01", "1e-1"),
        ("1e+01", "1e1"),
        ("1e+10", "1e10"),
        ("1.000e+00", "1e0"),
        ("1.100e+00", "1.1e0"),
    ],
)
def test_simplify(input, expected):
    assert FloatEncoder._simplify(input) == expected


@pytest.mark.parametrize(
    "input, expected",
    [
        (1, ("1.0", "1e0", "1.0")),
        (0.000_000_1, ("1e-07", "1e-7", "1e-7")),
        (10_000_000, ("10000000.0", "1e7", "1e7")),
        (156_000, ("156000.0", "1.56e5", "1.56e5")),
        (0.0056, ("0.0056", "5.6e-3", "0.0056")),
        (
            3.141592653589793,
            ("3.141592653589793", "3.141592653589793e0", "3.141592653589793"),
        ),
        (1e-15, ("1e-15", "1e-15", "1e-15")),
        (0.0, ("0.0", "0e0", "0.0")),
        (0, ("0.0", "0e0", "0.0")),
    ],
)
def test_encoders(input, expected):
    assert FloatEncoder.SIMPLE.encode(input) == expected[0]
    assert FloatEncoder.ENOTATION.encode(input) == expected[1]
    assert FloatEncoder.ENOTATION_IFF_SHORTER.encode(input) == expected[2]


# for reliable coverage, include a mandatory example for
# each branch in the test
@example(100.0)  # len('1e2') < len('100.0')
@example(1.0)  # len('1e0') == len('1.0')
@example(0.5)  # len('5e-1') > len('0.5')
@example(float("nan"))
@given(st.floats())
def test_strategies_comparison(r):
    s0 = FloatEncoder.SIMPLE.encode(r)
    s1 = FloatEncoder.ENOTATION.encode(r)
    s2 = FloatEncoder.ENOTATION_IFF_SHORTER.encode(r)

    if isnan(r):
        assert isnan(float(s1))
        assert isnan(float(s2))
    else:
        assert float(s1) == r
        assert float(s2) == r

    assert len(s2) <= len(s1)
    assert len(s2) <= len(s0)
    assert s2 in (s1, s0)

    if len(s1) < len(s0):
        assert s2 == s1
    else:
        assert s2 == s0



from pathlib import Path

import pytest

from inifix import format_string
from inifix.format import iniformat

DATA_DIR = Path(__file__).parent / "data"


def test_depr():
    body = (DATA_DIR / "format-in.ini").read_text(encoding="utf-8")
    s1 = format_string(body)
    with pytest.warns(
        DeprecationWarning,
        match="inifix.format.iniformat is deprecated",
    ):
        s2 = iniformat(body)
    assert s2 == s1



from importlib.metadata import version

import inifix


def test_dunder_version():
    assert inifix.__version__ == version("inifix")


def test_dunder_version_tuple():
    vt = inifix.__version_tuple__
    assert isinstance(vt, tuple)
    assert len(vt) == 3
    assert ".".join(str(i) for i in vt) == version("inifix")
